@@ -40,6 +40,7 @@
 #include <linux/videodev.h>
 
 #include "cheese-webcam.h"
+#include "cheese-flash.h"
 
 G_DEFINE_TYPE (CheeseWebcam, cheese_webcam, G_TYPE_OBJECT)
 
@@ -77,6 +78,7 @@ typedef struct
   GstElement *video_enc;
 
   GstElement *effect_filter, *csp_post_effect;
+  GstElement *video_balance, *csp_post_balance;
 
   gulong photo_handler_signal_id;
 
@@ -92,6 +94,8 @@ typedef struct
   int selected_device;
   CheeseVideoFormat *current_format;
 
+  CheeseFlash *flash;
+
   guint eos_timeout_id;
 } CheeseWebcamPrivate;
 
@@ -215,7 +219,6 @@ cheese_webcam_photo_data_cb (GstElement *element, GstBuffer *buffer,
 
   g_signal_handler_disconnect (G_OBJECT (priv->photo_sink),
                                priv->photo_handler_signal_id);
-  priv->photo_handler_signal_id = 0;
 
   g_signal_emit (webcam, webcam_signals[PHOTO_SAVED], 0);
 }
@@ -941,6 +944,16 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam, GError **error)
   {
     cheese_webcam_set_error_element_not_found (error, "ffmpegcolorspace");
   }
+  if ((priv->video_balance = gst_element_factory_make ("videobalance", "video_balance")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "videobalance");
+    return FALSE;
+  }
+  if ((priv->csp_post_balance = gst_element_factory_make ("ffmpegcolorspace", "csp_post_balance")) == NULL)
+  {
+    cheese_webcam_set_error_element_not_found(error, "ffmpegcolorspace");
+    return FALSE;
+  }
 
   if ((tee = gst_element_factory_make ("tee", "tee")) == NULL)
   {
@@ -976,11 +989,15 @@ cheese_webcam_create_video_display_bin (CheeseWebcam *webcam, GError **error)
     return FALSE;
 
   gst_bin_add_many (GST_BIN (priv->video_display_bin), priv->webcam_source_bin,
-                    priv->effect_filter, priv->csp_post_effect, tee, save_queue,
+                    priv->effect_filter, priv->csp_post_effect,
+                    priv->video_balance, priv->csp_post_balance,
+                    tee, save_queue,
                     video_display_queue, video_scale, video_sink, NULL);
 
   ok = gst_element_link_many (priv->webcam_source_bin, priv->effect_filter,
-                              priv->csp_post_effect, tee, NULL);
+                              priv->csp_post_effect, 
+                              priv->video_balance, priv->csp_post_balance,
+                              tee, NULL);
 
   ok &= gst_element_link_many (tee, save_queue, NULL);
   ok &= gst_element_link_many (tee, video_display_queue, video_scale, video_sink, NULL);
@@ -1344,16 +1361,11 @@ cheese_webcam_stop_video_recording (CheeseWebcam *webcam)
   }
 }
 
-gboolean
+void
 cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
-  if (priv->photo_handler_signal_id != 0) {
-    g_print ("Still waiting for previous photo data, ignoring new request\n");
-    return FALSE;
-  }
-
   g_free (priv->photo_filename);
   priv->photo_filename = g_strdup (filename);
 
@@ -1362,7 +1374,7 @@ cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
                                                     "handoff",
                                                     G_CALLBACK (cheese_webcam_photo_data_cb),
                                                     webcam);
-  return TRUE;
+  cheese_flash_fire (priv->flash);
 }
 
 static void
@@ -1537,7 +1549,8 @@ cheese_webcam_init (CheeseWebcam *webcam)
   priv->photo_filename      = NULL;
   priv->webcam_devices      = NULL;
   priv->device_name         = NULL;
-  priv->photo_handler_signal_id = 0;
+
+  priv->flash = cheese_flash_new ();
 }
 
 CheeseWebcam *
@@ -1704,3 +1717,33 @@ cheese_webcam_get_current_video_format (CheeseWebcam *webcam)
 
   return priv->current_format;
 }
+
+void
+cheese_webcam_get_balance_property_range (CheeseWebcam *webcam,
+                                          gchar *property,
+                                          gdouble *min, gdouble *max, gdouble *def)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+  GParamSpec *pspec;
+  
+  *min = 0.0;
+  *max = 0.0;
+  *def = 0.0;
+
+  pspec = g_object_class_find_property (
+    G_OBJECT_GET_CLASS (G_OBJECT (priv->video_balance)), property);
+
+  g_return_if_fail (pspec != NULL);
+
+  *min = G_PARAM_SPEC_DOUBLE (pspec)->minimum;
+  *max = G_PARAM_SPEC_DOUBLE (pspec)->maximum;
+  *def = G_PARAM_SPEC_DOUBLE (pspec)->default_value;
+}
+
+void
+cheese_webcam_set_balance_property (CheeseWebcam *webcam, gchar *property, gdouble value)
+{
+  CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
+
+  g_object_set (G_OBJECT (priv->video_balance), property, value, NULL);
+}
